import React, { ReactElement } from 'react'
import Link from 'next/link'
import useBasePath from '../../shared/use-base-path'

export type FooterOptions = {
  copyright: string
  logo: {
    img: string
    alt?: string
  }
  children?: ReactElement[]
}

export type LinkEntry = {
  href: string
  label: string
}

export type FooterColumnOptions = {
  title: string
  links: LinkEntry[]
}

export function FooterColumn({ title, links }: FooterColumnOptions) {
    return (
        <>
            <div>
                <h3 className="text-sm font-semibold text-gray-400 tracking-wider uppercase">
                    {title}
                </h3>
                <ul role="list" className="list-none m-0 p-0 mt-4 space-y-4">
                    {links.map(({ href, label }, key) => (
                        <li key={key}>
                            <Link href={href} passHref legacyBehavior>
                                <a className="text-base text-gray-500 hover:text-gray-900">
                                    {label}
                                </a>
                            </Link>
                        </li>
                    ))}
                </ul>
            </div>
        </>
    )
}

export function Footer({ logo, copyright, children }: FooterOptions) {
    const basePath = useBasePath()
    return (
        <footer className="bg-white" aria-labelledby="footer-heading">
            <h2 id="footer-heading" className="sr-only">
        Footer
            </h2>
            <div className="max-w-7xl mx-auto pt-12 pb-8 px-4">
                <div className="justify-items-center xl:grid xl:grid-cols-5 xl:gap-8">
                    <div className="space-y-4 xl:col-span-1">
                        <Link href='/'>
                            <img
                                className="h-20 w-auto hover:cursor-pointer"
                                src={`${basePath}${logo.img}`}
                                alt={logo.alt ? logo.alt : 'Logo'}
                            />
                        </Link>
                        <p className="text-gray-500 text-base">
              Fit für die digitale Verwaltung.
                            <br />
              Fit für die Zukunft.
                        </p>
                    </div>

                    <div className="mt-12 col-span-3 grid grid-cols-3 gap-16 xl:mt-0">
                        {children}
                    </div>
                </div>

                {copyright
                    ? (
                        <div className="mt-12 border-t border-gray-200 pt-6">
                            <p className="text-xs text-gray-400 text-center">
              Copyright &copy; {new Date().getFullYear()} {copyright}
                            </p>
                        </div>
                    )
                    : null}
                <p className="text-xs text-gray-400 text-center">
          * Es gilt die <a className="underline" href="https://fitko.de/datenschutz">Datenschutzerklärung der FITKO</a> mit der Maßgabe, dass kein Tracking durchgeführt wird und keine Cookies gesetzt werden.
                </p>
                <p className="text-xs text-gray-400 text-center">
          Wenn nicht anders vermerkt, sind die Inhalte dieser Webseite lizenziert unter der <a className="underline" href="https://creativecommons.org/licenses/by/4.0/deed.de">Creative
            Commons Namensnennung 4.0 International Public License (CC BY 4.0)</a>.
                </p>
                <p className="text-xs text-gray-400 text-center">
          Der Quellcode dieser Seite ist lizenziert unter der <a className="underline" href="https://git.fitko.de/fit-connect/entwicklungsportal/-/blob/main/LICENSE.md">EUPL</a> und
          findet sich <a className="underline" href="https://git.fitko.de/fit-connect/entwicklungsportal">im GitLab der FITKO</a>.
                </p>
                <p className="text-xs text-gray-400 text-center">
          Teile der Hintergrundgrafiken stammen aus dem <a className="underline" href="https://github.com/atlemo/SubtlePatterns">Subtle Patterns-Projekt</a>.
          Subtle Patterns ist lizenziert unter der <a className="underline" href="https://creativecommons.org/licenses/by-sa/3.0/">Creative Commons Attribution-ShareAlike 3.0 Unported License</a>.
                </p>
            </div>
        </footer>
    )
}
